#include "src/input/keyboard.hpp"

#include <Qt>
#include <map>

static std::map<int,KeyName> getKeymaps() {
	return std::map<int,KeyName>{
		{ Qt::Key_Space, { "space", "Space" } },
		{ Qt::Key_Exclam, { "exclaim", "!" } },
		{ Qt::Key_QuoteDbl, { "quotedbl", "\"" } },
		{ Qt::Key_NumberSign, { "hash", "#" } },
		{ Qt::Key_Dollar, { "dollar", "$" } },
		{ Qt::Key_Ampersand, { "ampersand", "&" } },
		{ Qt::Key_Apostrophe, { "quote", "'" } },
		{ Qt::Key_ParenLeft, { "leftparen", "(" } },
		{ Qt::Key_ParenRight, { "rightparen", ")" } },
		{ Qt::Key_Asterisk, { "asterisk", "*" } },
		{ Qt::Key_Plus, { "plus", "+" } },
		{ Qt::Key_Comma, { "comma", "," } },
		{ Qt::Key_Minus, { "minus", "-" } },
		{ Qt::Key_Period, { "period", "." } },
		{ Qt::Key_Slash, { "slash", "/" } },
		{ Qt::Key_0, { "num0", "0" } },
		{ Qt::Key_1, { "num1", "1" } },
		{ Qt::Key_2, { "num2", "2" } },
		{ Qt::Key_3, { "num3", "3" } },
		{ Qt::Key_4, { "num4", "4" } },
		{ Qt::Key_5, { "num5", "5" } },
		{ Qt::Key_6, { "num6", "6" } },
		{ Qt::Key_7, { "num7", "7" } },
		{ Qt::Key_8, { "num8", "8" } },
		{ Qt::Key_9, { "num9", "9" } },
		{ Qt::Key_Colon, { "colon", ":" } },
		{ Qt::Key_Semicolon, { "semicolon", ";" } },
		{ Qt::Key_Less, { "less", "<" } },
		{ Qt::Key_Equal, { "equals", "=" } },
		{ Qt::Key_Greater, { "greater", ">" } },
		{ Qt::Key_Question, { "question", "?" } },
		{ Qt::Key_At, { "at", "@" } },
		{ Qt::Key_A, { "a", "A" } },
		{ Qt::Key_B, { "b", "B" } },
		{ Qt::Key_C, { "c", "C" } },
		{ Qt::Key_D, { "d", "D" } },
		{ Qt::Key_E, { "e", "E" } },
		{ Qt::Key_F, { "f", "F" } },
		{ Qt::Key_G, { "g", "G" } },
		{ Qt::Key_H, { "h", "H" } },
		{ Qt::Key_I, { "i", "I" } },
		{ Qt::Key_J, { "j", "J" } },
		{ Qt::Key_K, { "k", "K" } },
		{ Qt::Key_L, { "l", "L" } },
		{ Qt::Key_M, { "m", "M" } },
		{ Qt::Key_N, { "n", "N" } },
		{ Qt::Key_O, { "o", "O" } },
		{ Qt::Key_P, { "p", "P" } },
		{ Qt::Key_Q, { "q", "Q" } },
		{ Qt::Key_R, { "r", "R" } },
		{ Qt::Key_S, { "s", "S" } },
		{ Qt::Key_T, { "t", "T" } },
		{ Qt::Key_U, { "u", "U" } },
		{ Qt::Key_V, { "v", "V" } },
		{ Qt::Key_W, { "w", "W" } },
		{ Qt::Key_X, { "x", "X" } },
		{ Qt::Key_Y, { "y", "Y" } },
		{ Qt::Key_Z, { "z", "Z" } },
		{ Qt::Key_BracketLeft, { "leftbracket", "[" } },
		{ Qt::Key_Backslash, { "backslash", "\\" } },
		{ Qt::Key_BracketRight, { "rightbracket", "]" } },
		{ Qt::Key_AsciiCircum, { "caret", "^" } },
		{ Qt::Key_Underscore, { "underscore", "_" } },
		{ Qt::Key_QuoteLeft, { "backquote", "`" } },
		{ Qt::Key_AsciiTilde, { "tilde", "~" } },
		{ Qt::Key_Escape, { "escape", "Esc" } },
		{ Qt::Key_Tab, { "tab", "Tab" } },
		{ Qt::Key_Backspace, { "backspace", "Backspace" } },
		{ Qt::Key_Return, { "enter", "Enter" } },
		{ Qt::Key_Enter, { "kp_enter", "Numpad Enter" } },
		{ Qt::Key_Insert, { "insert", "Insert" } },
		{ Qt::Key_Delete, { "del", "Delete" } },
		{ Qt::Key_Clear, { "clear", "Clear" } },
		{ Qt::Key_Pause, { "pause", "Pause Break" } },
		{ Qt::Key_Print, { "print_screen", "Print Screen" } },
		{ Qt::Key_SysReq, { "sysreq", "SysReq" } },
		{ Qt::Key_Home, { "home", "Home" } },
		{ Qt::Key_End, { "end", "End" } },
		{ Qt::Key_Left, { "left", "Left Arrow" } },
		{ Qt::Key_Up, { "up", "Up Arrow" } },
		{ Qt::Key_Right, { "right", "Right Arrow" } },
		{ Qt::Key_Down, { "down", "Down Arrow" } },
		{ Qt::Key_PageUp, { "pageup", "Page Up" } },
		{ Qt::Key_PageDown, { "pagedown", "Page Down" } },
		{ Qt::Key_Shift, { "shift", "Shift" } },
		{ Qt::Key_Control, { "ctrl", "Ctrl" } },
		{ Qt::Key_CapsLock, { "capslock", "Caps Lock" } },
		{ Qt::Key_NumLock, { "numlock", "Num Lock" } },
		{ Qt::Key_ScrollLock, { "scroll_lock", "Scroll Lock" } },
		{ Qt::Key_F1, { "f1", "F1" } },
		{ Qt::Key_F2, { "f2", "F2" } },
		{ Qt::Key_F3, { "f3", "F3" } },
		{ Qt::Key_F4, { "f4", "F4" } },
		{ Qt::Key_F5, { "f5", "F5" } },
		{ Qt::Key_F6, { "f6", "F6" } },
		{ Qt::Key_F7, { "f7", "F7" } },
		{ Qt::Key_F8, { "f8", "F8" } },
		{ Qt::Key_F9, { "f9", "F9" } },
		{ Qt::Key_F10, { "f10", "F10" } },
		{ Qt::Key_F11, { "f11", "F11" } },
		{ Qt::Key_F12, { "f12", "F12" } },
		{ Qt::Key_F13, { "f13", "F13" } },
		{ Qt::Key_F14, { "f14", "F14" } },
		{ Qt::Key_F15, { "f15", "F15" } },
		{ Qt::KeypadModifier | Qt::Key_Asterisk, { "multiply", "Numpad *" } },
		{ Qt::KeypadModifier | Qt::Key_Plus, { "add", "Numpad +" } },
		{ Qt::KeypadModifier | Qt::Key_Minus, { "subtract", "Numpad -" } },
		{ Qt::KeypadModifier | Qt::Key_Period, { "kp_period", "Numpad ." } },
		{ Qt::KeypadModifier | Qt::Key_Slash, { "divide", "Numpad /" } },
		{ Qt::KeypadModifier | Qt::Key_0, { "keypad0", "Numpad 0" } },
		{ Qt::KeypadModifier | Qt::Key_1, { "keypad1", "Numpad 1" } },
		{ Qt::KeypadModifier | Qt::Key_2, { "keypad2", "Numpad 2" } },
		{ Qt::KeypadModifier | Qt::Key_3, { "keypad3", "Numpad 3" } },
		{ Qt::KeypadModifier | Qt::Key_4, { "keypad4", "Numpad 4" } },
		{ Qt::KeypadModifier | Qt::Key_5, { "keypad5", "Numpad 5" } },
		{ Qt::KeypadModifier | Qt::Key_6, { "keypad6", "Numpad 6" } },
		{ Qt::KeypadModifier | Qt::Key_7, { "keypad7", "Numpad 7" } },
		{ Qt::KeypadModifier | Qt::Key_8, { "keypad8", "Numpad 8" } },
		{ Qt::KeypadModifier | Qt::Key_9, { "keypad9", "Numpad 9" } },
		{ Qt::KeypadModifier | Qt::Key_Equal, { "kp_equals", "Numpad =" } },
		{ Qt::KeypadModifier | Qt::Key_Return, { "kp_enter", "Numpad Enter" } },
		{ Qt::KeypadModifier | Qt::Key_Enter, { "kp_enter", "Numpad Enter" } }
	};
};

const KeyName &Keycode::getNames( int key ) {
	static const std::map<int,KeyName> s_keymap = getKeymaps();
	static const KeyName s_nokey = { "nul", "" };

	const auto names = s_keymap.find( key );
	return (names == s_keymap.end()) ? s_nokey : names->second;
}
