#ifndef SRC_RHDC_WEB_API_HPP_
#define SRC_RHDC_WEB_API_HPP_

#include <unordered_map>
#include <functional>
#include <vector>
#include "src/types.hpp"
#include "src/rhdc/core/psl.hpp"
#include "src/rhdc/core/hack.hpp"
#include "src/rhdc/core/credentials.hpp"

enum class HttpStatusCode : int {
	None = 0,

	OK = 200,
	Created = 201,
	Accepted = 202,
	NoContent = 204,

	BadRequest = 400,
	Unauthorized = 401,
	Forbidden = 403,
	NotFound = 404,
	Gone = 410,
	TooManyRequests = 429,

	InternalServerError = 500,
	BadGateway = 502,
	ServiceUnavailable = 503,
	GatewayTimeout = 504
};

enum class ApiErrorType {
	NotAuthorized,
	RegistrationRequired,
	NotFound,
	ServerDown,
	JsonError,
	UnknownError
};

struct Starpower {
	RhdcHackProgress progress;
	int64 playTime;
};

struct FollowedHack {
	RhdcHackInfo info;
	string downloadUrl;
	std::vector<string> versions;
	std::vector<string> groups;
};

namespace RhdcApi {

	/* login.cpp */

	extern bool isAuthenticated() noexcept;

	extern void loginAsync(
		const RhdcCredentials &credentials,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void logout() noexcept;

	/* games.cpp */

	extern void getFollowedHacksAsync(
		const std::function<void(HashMap<string,FollowedHack>&)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void addHackToListAsync(
		const string &hackId,
		const string &group,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void removeHackFromListAsync(
		const string &hackId,
		const string &group,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void deleteListAsync(
		const string &group,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	/* starpower.cpp */

	extern void getStarpowerAsync(
		const std::function<void(HashMap<string,Starpower>&)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void getStarpowerAsync(
		const string &hackId,
		const std::function<void(Starpower&)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void submitRatingsAsync(
		const string &hackId,
		ubyte qualityRating,
		ubyte difficultyRating,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void submitStarProgressAsync(
		const string &hackId,
		ushort starsCollected,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void submitCompletionAsync(
		const string &hackId,
		bool isComplete,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	extern void submitPlayTimeAsync(
		const string &hackId,
		int64 playTime,
		const std::function<void(void)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

	/* layouts.cpp */

	extern void fetchLayoutsAysnc(
		const std::function<void(LayoutManager&)> &onSuccess,
		const std::function<void(ApiErrorType)> &onFailure
	);

};



#endif /* SRC_RHDC_WEB_API_HPP_ */
